var app = {
    init: function () {
        app.setDefaultState();
        app.bindListeners();
        app.bindEvents();
        app.removeTempResolution();
    },

    bindEvents: function () {
        app.bindDeviceEvents();
        app.bindRecordEvents();
        app.bindTabsEvents();
    },

    bindDeviceEvents: function () {
        $('.mic').on('click', function (e) {
            if ($(this).hasClass("disabled")) return;
            var icon = $(this).find("i");
            var enabled = icon.hasClass("icon-mic-on");
            if (enabled) {
                icon.removeClass("icon-mic-on").addClass("icon-mic-off");
                $("#audio-inputs").attr("disabled", true);
                $("#audio-indicator").hide();
            } else {
                icon.removeClass("icon-mic-off").addClass("icon-mic-on");
                $("#audio-inputs").attr("disabled", false);
                $("#audio-indicator").show();
            };

            app.setStorage("microphoneEnabled", !enabled);
        });

        $('.camera').on('click', function (e) {
            if ($(this).hasClass("disabled")) return;
            var icon = $(this).find("i");
            var enabled = icon.hasClass("icon-webcam-on");
            if (enabled) {
                icon.removeClass("icon-webcam-on").addClass("icon-webcam-off");
                $("#video-inputs").attr("disabled", true)
            } else {
                icon.removeClass("icon-webcam-off").addClass("icon-webcam-on");
                $("#video-inputs").attr("disabled", false)
            }
            app.setStorage("cameraEnabled", !enabled);
            setTimeout(function () {
                var tab = $('.tab-action.active').attr("data-action");
                chrome.extension.sendMessage({
                    action: 'camera_clicked',
                    enabled: !enabled,
                    tab: tab
                });
            }, 300)
        });

        $("#video-inputs").on("change", function () {
            app.setStorage("video-input", $(this).val());
            chrome.extension.sendMessage({
                action: 'video_input_changed'
            });
        })

        $("#audio-inputs").on("change", function () {
            app.setStorage("audio-input", $(this).val());
        })

        $("#video-resolution").on("change", function () {
            app.setStorage("video-resolution", $(this).val());
        })

        $(".allow-perm").on("click", function () {
            window.open('https://shazoom.io/shazoom-extension/setup?disable_editor=y', "_blank");
        });

        $('.refresh-ext').on("click", function onRefresh() {
            app.checkDevicePermissions();
        });

        $('.welcome-text_got-it').on("click", function hideText() {
            app.setStorage('welcome-text', 'hidden');
            $(this).closest('.welcome-text').addClass('hidden');
        })
		
		app.checkDevicePermissions();
    },

    bindRecordEvents: function () {
        $('.record-popup').on('click', function (e) {
            chrome.storage.sync.get(null, function (items) {
                var type = $('.source .tab-action.active').attr("data-action");
                var gotIt = items["gotIt"] === true ? true : false;
                if (gotIt || type === "webcam" || (items.cameraPermitted && items.microphonePermitted)) {
				//if ((items.cameraPermitted && items.microphonePermitted) || gotIt || type === "webcam") {
                    
                    var audioEnabled = $(".mic i").hasClass("icon-mic-off") || $(".mic").hasClass("disabled") ? false : true;
                    
                    chrome.runtime.sendMessage({
                        action: 'capture',
                        type: type,
                        audio: audioEnabled
                    }, function () {
                        app.closePopup();
                    });
                } else {
                    chrome.runtime.sendMessage({
                        action: 'gotIt'
                    });
                    app.closePopup();
                }
            })
        });

    },

    bindTabsEvents: function () {
        $('.tab-action').on('click', function (e) {
            var old_active = $('.tab-action.active').attr("data-action");
            if ($(this).hasClass("active")) return;
            $('.tab-action').removeClass("active");
            $(this).addClass("active");
            var tab = $(this).attr("data-action");
            app.setStorage("tab", tab);
            if (old_active == tab || (old_active === "tab" && tab === "screen") || (old_active === "screen" && tab === "tab")) return;
            var webcamEnabled = !$('.camera').hasClass("disabled") && $('.camera i').hasClass("icon-webcam-on");
            chrome.extension.sendMessage({
                action: 'tab_clicked',
                tab: tab,
                webCam: webcamEnabled
            });
        });
    },

    bindListeners: function () {
        chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
            if (request.key == "setDevicePermissions") {
				 var perm = {};
					perm.hasMicrophone = request.hasMicrophone;
					perm.isWebsiteHasMicrophonePermissions = request.isWebsiteHasMicrophonePermissions;
					perm.hasWebcam = request.hasWebcam;
					perm.isWebsiteHasWebcamPermissions = request.isWebsiteHasWebcamPermissions;
					perm.audioInputs = request.audioInputs;
					perm.videoInputs = request.videoInputs;
				//var perm = Object.assign({}, request);
                app.setDevicePermissions(perm)
            }
        });
    },
    removeTempResolution: function() {
        // TODO:
        $('.video-resolution.hidden').remove();
    },

    showError: function (target, type, errmsg) {
        app.hideError(target);
        var errorText;
        if (type == 'email') {
            errorText = "Invalid email"
        } else if (type == "password") {
            errorText = "Incorrect password";
        } else if (type == 'required') {
            errorText = "Required";
        } else if (type == undefined) {
            errorText = errmsg;
        }
        $(target).before('<div class="fb-msg danger">' + errorText + '</div>')
    },

    hideError: function (target) {
        $(target).prev('.danger').remove()
    },
  /*   showRecordingBlock: function () {
        $(".capture-box").removeClass("hiddenRecorder");
        $(".settings-box").addClass("hide");
        $(".header").hide();
        $(".login-header").hide();
        $(".register-header").hide();
        $(".settings-header").hide();
        app.checkDevicePermissions();
    }, */

    showIncognitoBox: function () {
        $(".incognito-box").removeClass("hide");    
    },

    showExtensionBox: function () {
        $(".extension-box").removeClass("hide");
    },

    checkDevicePermissions: function () {
        chrome.extension.sendMessage({
            action: 'checkDevicePermissions'
        }, function (response) {
			if (chrome.runtime.lastError) {
                                    return;
			} else {
				console.log({response});
			}
            
         });
    },

    setDevicePermissions: function (perm) {
        if (!perm) return;
        if (typeof perm.hasMicrophone != "undefined") {
            app.disableMicrophone(perm.hasMicrophone);
        }
        if (typeof perm.hasWebcam != "undefined") {
            app.disableWebCam(perm.hasWebcam);
        }
        var videoDeviceExist = false;
		
        if (typeof perm.videoInputs != "undefined") {
            if (perm.videoInputs.length) {
                var htm = "";
                for (var i = 0; i < perm.videoInputs.length; i++) {					
                    if (perm.videoInputs[i].label.toLowerCase().indexOf("https") === -1) {
                        videoDeviceExist = true;
                        htm += "<option value='" + perm.videoInputs[i].id + "'>" + perm.videoInputs[i].label + "</option>"
                    }
                }
                $("#video-inputs").html(htm);
                if (!perm.hasWebcam || !perm.isWebsiteHasWebcamPermissions) {
                    $("#video-inputs").attr('disabled', true)
                }
                setTimeout(function () {
                    if (perm.hasWebcam && perm.isWebsiteHasWebcamPermissions) {
                        app.showWebCamPreview();
                    }
                }, 200)
            } else {
                $("#video-inputs").html("<option>No video source</option>").attr("disabled", true);
                app.disableWebCam(perm.hasWebcam);
            }
            if (!videoDeviceExist) {
				var icon =  $('.camera').find("i");
                icon.removeClass("icon-webcam-on").addClass("icon-webcam-off");
                $('.camera').attr("disabled", true);
                $("#video-inputs").html("<option>No video source</option>").attr("disabled", true);
            }
        }
        var audioDeviceExist = false;
        if (typeof perm.audioInputs != "undefined") {
            if (perm.audioInputs.length) {
                var htm = "";
                for (var i = 0; i < perm.audioInputs.length; i++) {
                    if (perm.audioInputs[i].label.toLowerCase().indexOf("https") === -1) {
                        audioDeviceExist = true;
                        htm += "<option value='" + perm.audioInputs[i].id + "'>" + perm.audioInputs[i].label + "</option>"
                    }
                }
                $("#audio-inputs").html(htm);
                if (!perm.hasMicrophone || !perm.isWebsiteHasMicrophonePermissions) {
                    $("#audio-inputs").attr('disabled', true)
                }

                
            } else {
                $("#audio-inputs").html("<option>No audio source</option>").attr("disabled", true);
            }
            if (!audioDeviceExist) {
				var icon =  $('.mic').find("i");
                icon.removeClass("icon-mic-on").addClass("icon-mic-off");
                $('.mic').off('click');
                $("#audio-inputs").html("<option>No audio source</option>").attr("disabled", true);
            }
        }

        // if(!perm.microphone)
        //     app.requestDeviceAccess();
        if ((perm.isWebsiteHasWebcamPermissions || !perm.hasWebcam) && (perm.isWebsiteHasMicrophonePermissions || !perm.hasMicrophone)) {
            $(".allow-perm").hide();
        }

        chrome.storage.sync.get(null, function (items) {
            if (items["video-input"] && videoDeviceExist) {
                $("#video-inputs").val(items["video-input"])
            }

            if (items["audio-input"] && audioDeviceExist) {
                $("#audio-inputs").val(items["audio-input"])
            }

            if (items["video-resolution"]) {
                $("#video-resolution").val(items["video-resolution"])
            }
            if(items["welcome-text"]) {
                $(".welcome-text").addClass('hidden');
            }
        })

    },

    requestDeviceAccess: function () {
        chrome.tabs.query(
            { active: true, currentWindow: true },
            function (tabs) {
                chrome.tabs.sendMessage(tabs[0].id, { key: "requestDeviceAccess" }, function (response) { 
					if (chrome.runtime.lastError) {
						return;
					} else {}				
				});
            }
        );
    },

    disableMicrophone: function (enable) {
        app.setStorage("microphonePermitted", enable ? enable : false);
        if (enable) {
            $('.mic').removeClass("disabled");
        } else {
            $('.mic').addClass("disabled");
        }

        if (!enable) {
            $("#audio-indicator").remove();
        } else {
            chrome.storage.sync.get(null, function (items) {
                app.createAudioEqualizer(items['audio-input']);
            })
        }
    },

    disableWebCam: function (enable) {
        app.setStorage("cameraPermitted", !!enable);
        if (enable) {
            $('.camera').removeClass("disabled");
        } else {
            $('.camera').addClass("disabled");
        }
    },

    showWebCamPreview: function () {
        chrome.storage.sync.get(null, function (items) {
            var activeTab = items["tab"] || "tab";
            var cameraEnabled = items["cameraEnabled"] === false ? false : true;
           /*  if (activeTab === "webcam") {
                chrome.extension.sendMessage({
                    action: 'webcamPreview'
                });
            } else  */if (cameraEnabled) {
                chrome.extension.sendMessage({
                    action: 'smallwebcamPreview',
                    type: activeTab
                });
            }
        })
    },

    setStorage: function (key, value) {
        var params = {};
        params[key] = value;
        chrome.storage.sync.set(params, function () { });
    },

    setDefaultState: function () {
        chrome.storage.sync.get(null, function (items) {
            var microphoneEnabled = items["microphoneEnabled"] === false ? false : true;
            var cameraEnabled = items["cameraEnabled"] === false ? false : true;
            var activeTab = items["tab"] || "tab";
            var highlightCursor = items["highlightCursor"] || "off";
            var realtimeMarker = items["realtimeMarker"] === false ? false : true;
            var pagesName = items["pagesName"] || "";
            $(".tab-action").removeClass("active");
            $(".tab-action[data-action='" + activeTab + "']").addClass("active");
            var icon = $(".mic").find("i");
            if (microphoneEnabled) {
                icon.removeClass("icon-mic-off").addClass("icon-mic-on");
                $("#audio-indicator").show();
            } else {
                icon.removeClass("icon-mic-on").addClass("icon-mic-off");
                $("#audio-inputs").attr("disabled", true);
                $("#audio-indicator").hide();
            };
            icon = $(".camera").find("i");
            if (cameraEnabled) {
                icon.removeClass("icon-webcam-off").addClass("icon-webcam-on");
            } else {
                icon.removeClass("icon-webcam-on").addClass("icon-webcam-off");
                $("#video-inputs").attr("disabled", true)
            }

            $("input[name='highlightCursor']").prop("checked", true);
            $("input[name='highlightCursor'][value='" + highlightCursor + "']").prop("checked", true);
            $("#realtime-marker").prop("checked", realtimeMarker);
            $("#pages-name").val(pagesName);
        })
    },

    closePopup: function () {
        window.close();
        chrome.browserAction.setPopup({ popup: '' });
    },

    createAudioEqualizer: function (sourceId) {
        var canvas = document.getElementById("audio-indicator");
        if (canvas == null) return;
        window.AudioContext = window.AudioContext || window.webkitAudioContext, audioContext = new AudioContext();
        var audioOptions = sourceId ? { optional: [{ sourceId: sourceId }] } : true;
        navigator.mediaDevices.getUserMedia({
            audio: audioOptions,
            video: false
        })
            .then(function (e) {
                var o = audioContext.createMediaStreamSource(e);
                app.meter = app.createAudioMeter(audioContext);
                o.connect(app.meter);
                app.drawMeter(app.meter)

            })
            .catch(function (e) {
                console.log("catch", e);
                canvas.style.display = "none";
            })
    },

    drawMeter: function (e) {
        var canvasContext = $("#audio-indicator")[0].getContext("2d");
        var canvasHeight = $("#audio-indicator")[0].height;
        var canvasWidth = $("#audio-indicator")[0].width;
        var i = canvasWidth / 2 - 65,
            o = canvasHeight,
            t = -app.meter.volume * canvasHeight * 5;
        canvasContext.clearRect(0, 0, canvasWidth, canvasHeight), canvasContext.fillStyle = "rgba(0, 0, 0, 0.2)", canvasContext.strokeStyle = "red", app.roundRect(canvasContext, i - 10, o - canvasHeight, 150, canvasHeight, 50, "rgba(200,200,200,0.1)", "rgba(0,0,0,0.2)", 20), canvasContext.fillStyle = "black", canvasContext.fillRect(i, o, 130, -10), app.meter.checkClipping() && (canvasContext.fillStyle = "red");
        var n = app.map(Math.abs(t), 0, 200, 50, 255);
        canvasContext.fillStyle = "rgb(" + Math.floor(n) + "," + Math.floor(205 - n / 2) + ",50)", canvasContext.fillRect(i, o, 130, t), window.requestAnimationFrame(app.drawMeter)
    },

    createAudioMeter: function (audioContext, clipLevel, averaging, clipLag) {
        var processor = audioContext.createScriptProcessor(512);
        processor.onaudioprocess = app.volumeAudioProcess;
        processor.clipping = false;
        processor.lastClip = 0;
        processor.volume = 0;
        processor.clipLevel = clipLevel || 0.98;
        processor.averaging = averaging || 0.95;
        processor.clipLag = clipLag || 750;

        // this will have no effect, since we don't copy the input to the output,
        // but works around a current Chrome bug.
        processor.connect(audioContext.destination);

        processor.checkClipping =
            function () {
                if (!this.clipping)
                    return false;
                if ((this.lastClip + this.clipLag) < window.performance.now())
                    this.clipping = false;
                return this.clipping;
            };

        processor.shutdown =
            function () {
                this.disconnect();
                this.onaudioprocess = null;
            };

        return processor;
    },

    volumeAudioProcess: function (event) {
        var buf = event.inputBuffer.getChannelData(0);
        var bufLength = buf.length;
        var sum = 0;
        var x;

        // Do a root-mean-square on the samples: sum up the squares...
        for (var i = 0; i < bufLength; i++) {
            x = buf[i];
            if (Math.abs(x) >= this.clipLevel) {
                this.clipping = true;
                this.lastClip = window.performance.now();
            }
            sum += x * x;
        }

        // ... then take the square root of the sum.
        var rms = Math.sqrt(sum / bufLength);

        // Now smooth this out with the averaging factor applied
        // to the previous sample - take the max here because we
        // want "fast attack, slow release."
        this.volume = Math.max(rms, this.volume * this.averaging);
    },

    roundRect: function (e, i, o, t, n, a, r, c, s) {
        var d = void 0 === c || 0 !== c;
        a = void 0 === a ? 5 : a, e.save(), e.beginPath(), e.moveTo(i + a, o), e.lineTo(i + t - a, o), e.quadraticCurveTo(i + t, o, i + t, o + a), e.lineTo(i + t, o + n - a), e.quadraticCurveTo(i + t, o + n, i + t - a, o + n), e.lineTo(i + a, o + n), e.quadraticCurveTo(i, o + n, i, o + n - a), e.lineTo(i, o + a), e.quadraticCurveTo(i, o, i + a, o), e.closePath(), e.lineWidth = s, d && 0 !== c && (e.strokeStyle = void 0 === c ? "#000" : c, e.stroke()), r && (e.fillStyle = r, e.fill()), e.restore()
    },

    map: function (e, i, o, t, n) {
        return e > o ? n : e < i ? t : (e - i) * (n - t) / (o - i) + t
    }
};

$(document).ready(function () {
    app.init();
    setTimeout(function () {
        $('body').css('overflow', 'overlay');
    }, 300);
})

